@@ -92,9 +92,9 @@ end
 #
 # === Idle Timeout
 #
-# If a connection hasn't been used for 5 seconds it will automatically be
+# If a connection hasn't been used for this number of seconds it will automatically be
 # reset upon the next use to avoid attempting to send to a closed connection.
-# This can be adjusted through idle_timeout.
+# The default value is 5 seconds. nil means no timeout. Set through #idle_timeout.
 #
 # Reducing this value may help avoid the "too many connection resets" error
 # when sending non-idempotent requests while increasing this value will cause
@@ -110,14 +110,6 @@ end
 # The amount of time to wait for a connection to be opened.  Set through
 # #open_timeout.
 #
-# === Idle Timeout
-#
-# If a connection has not been used in this many seconds it will be reset when
-# a request would use the connection.  The default idle timeout is unlimited.
-# If you know the server's idle timeout setting this value will eliminate
-# failures from attempting non-idempotent requests on closed connections.  Set
-# through #idle_timeout.
-#
 # === Socket Options
 #
 # Socket options may be set on newly-created connections.  See #socket_options
@@ -161,6 +153,18 @@ end
 # The method of determining if the resource was created or not is unique to
 # the particular service you are using.  Of course, you will want to add
 # protection from infinite looping.
+#
+# === Connection Termination
+#
+# If you are done using the Net::HTTP::Persistent instance you may shut down
+# all the connections in the current thread with #shutdown.  This is not
+# recommended for normal use, it should only be used when it will be several
+# minutes before you make another HTTP request.
+#
+# If you are using multiple threads, call #shutdown in each thread when the
+# thread is done making requests.  If you don't call shutdown, that's OK.
+# Ruby will automatically garbage collect and shutdown your HTTP connections
+# when the thread terminates.
 
 class Net::HTTP::Persistent
 
@@ -172,7 +176,7 @@ class Net::HTTP::Persistent
   ##
   # The version of Net::HTTP::Persistent you are using
 
-  VERSION = '2.5.2'
+  VERSION = '2.7'
 
   ##
   # Error class for errors raised by Net::HTTP::Persistent.  Various
@@ -400,7 +404,7 @@ class Net::HTTP::Persistent
 
     @generation         = 0 # incremented when proxy URI changes
     @ssl_generation     = 0 # incremented when SSL session variables change
-    @reuse_ssl_sessions = true
+    @reuse_ssl_sessions = OpenSSL::SSL.const_defined? :Session
 
     @retry_change_requests = false
 
@@ -496,8 +500,7 @@ class Net::HTTP::Persistent
       connection = connections[connection_id]
       ssl connection if use_ssl
     else
-      last_used = Thread.current[@timeout_key][connection.object_id]
-      reset connection unless last_used > max_age
+      reset connection if expired? connection
     end
 
     unless connection.started? then
@@ -544,6 +547,19 @@ class Net::HTTP::Persistent
     CGI.escape str if str
   end
 
+  ##
+  # Returns true if the connection should be reset due to an idle timeout,
+  # false otherwise.
+
+  def expired? connection
+    return false unless @idle_timeout
+    return true  if     @idle_timeout.zero?
+
+    last_used = Thread.current[@timeout_key][connection.object_id]
+
+    Time.now - last_used > @idle_timeout
+  end
+
   ##
   # Finishes the Net::HTTP +connection+
 
@@ -623,9 +639,11 @@ class Net::HTTP::Persistent
   end
 
   ##
-  # If a connection hasn't been used since max_age it will be reset and reused
+  # Deprecated in favor of #expired?
+
+  def max_age # :nodoc:
+    return Time.now + 1 unless @idle_timeout
 
-  def max_age
     Time.now - @idle_timeout
   end
 
