@@ -121,7 +121,7 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   end
 
   def reqs
-    Thread.current[@http.request_key] ||= {}
+    Thread.current[@http.request_key] ||= Hash.new 0
   end
 
   def ssl_conns
@@ -141,6 +141,10 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
   def test_initialize
     assert_nil @http.proxy_uri
+
+    ssl_session_exists = OpenSSL::SSL.const_defined? :Session
+
+    assert_equal ssl_session_exists, @http.reuse_ssl_sessions
   end
 
   def test_initialize_name
@@ -148,6 +152,21 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_equal 'name', http.name
   end
 
+  def test_initialize_no_ssl_session
+    skip "OpenSSL::SSL::Session does not exist on #{RUBY_PLATFORM}" unless
+      OpenSSL::SSL.const_defined? :Session
+
+    ssl_session = OpenSSL::SSL::Session
+
+    OpenSSL::SSL.send :remove_const, :Session
+
+    http = Net::HTTP::Persistent.new
+
+    refute http.reuse_ssl_sessions
+  ensure
+    OpenSSL::SSL.const_set :Session, ssl_session if ssl_session
+  end
+
   def test_initialize_proxy
     proxy_uri = URI.parse 'http://proxy.example'
 
@@ -251,6 +270,60 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_same c, conns[0]['example.com:80']
   end
 
+  def test_connection_for_cached_expire_always
+    cached = basic_connection
+    cached.start
+    conns[0]['example.com:80'] = cached
+    reqs[cached.object_id] = 10
+    touts[cached.object_id] = Time.now # last used right now
+
+    @http.idle_timeout = 0
+
+    c = @http.connection_for @uri
+
+    assert c.started?
+
+    assert_same cached, c
+
+    assert_equal 0, reqs[cached.object_id],
+                 'connection reset due to timeout'
+  end
+
+  def test_connection_for_cached_expire_never
+    cached = basic_connection
+    cached.start
+    conns[0]['example.com:80'] = cached
+    reqs[cached.object_id] = 10
+    touts[cached.object_id] = Time.now # last used right now
+
+    @http.idle_timeout = nil
+
+    c = @http.connection_for @uri
+
+    assert c.started?
+
+    assert_same cached, c
+
+    assert_equal 10, reqs[cached.object_id],
+                 'connection reset despite no timeout'
+  end
+
+  def test_connection_for_cached_expired
+    cached = basic_connection
+    cached.start
+    conns[0]['example.com:80'] = cached
+    reqs[cached.object_id] = 10
+    touts[cached.object_id] = Time.now - 3600
+
+    c = @http.connection_for @uri
+
+    assert c.started?
+
+    assert_same cached, c
+    assert_equal 0, reqs[cached.object_id],
+                 'connection not reset due to timeout'
+  end
+
   def test_connection_for_finished_ssl
     uri = URI.parse 'https://example.com/path'
     c = @http.connection_for uri
@@ -412,7 +485,7 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     c = @http.connection_for @uri
 
     assert c.started?
-    assert_nil reqs[c.object_id]
+    assert_equal 0, reqs[c.object_id]
 
     assert_same cached, c
   end
@@ -433,6 +506,26 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert_equal '+%3F', @http.escape(' ?')
   end
 
+  def test_expired_eh
+    c = basic_connection
+    touts[c.object_id] = Time.now - 11
+
+    @http.idle_timeout = 0
+    assert @http.expired? c
+
+    @http.idle_timeout = 10
+    assert @http.expired? c
+
+    @http.idle_timeout = 11
+    assert @http.expired? c
+
+    @http.idle_timeout = 12
+    refute @http.expired? c
+
+    @http.idle_timeout = nil
+    refute @http.expired? c
+  end
+
   def test_finish
     c = basic_connection
     reqs[c.object_id] = 5
@@ -441,7 +534,7 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
     refute c.started?
     assert c.finished?
-    assert_nil reqs[c.object_id]
+    assert_equal 0, reqs[c.object_id]
   end
 
   def test_finish_io_error
@@ -478,6 +571,10 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
   def test_max_age
     assert_in_delta Time.now - 5, @http.max_age
+
+    @http.idle_timeout = nil
+
+    assert_in_delta Time.now + 1, @http.max_age
   end
 
   def test_normalize_uri
@@ -693,6 +790,10 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   def test_request_close_1_0
     c = connection
 
+    class << c
+      remove_method :request
+    end
+
     def c.request req
       @req = req
       r = Net::HTTPResponse.allocate
@@ -740,6 +841,10 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   def test_request_connection_close_response
     c = connection
 
+    class << c
+      remove_method :request
+    end
+
     def c.request req
       @req = req
       r = Net::HTTPResponse.allocate
@@ -887,7 +992,7 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
     assert c.started?
     assert c.finished?
     assert c.reset?
-    assert_nil reqs[c.object_id]
+    assert_equal 0, reqs[c.object_id]
     assert_equal Net::HTTP::Persistent::EPOCH, touts[c.object_id]
   end
 
@@ -948,7 +1053,6 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
   def test_shutdown
     ssl_conns
     c = connection
-    cs = conns
     rs = reqs
     ts = touts
 
@@ -1177,7 +1281,6 @@ class TestNetHttpPersistent < MiniTest::Unit::TestCase
 
   def test_ssl_cleanup
     uri1 = URI.parse 'https://one.example'
-    uri2 = URI.parse 'https://two.example'
 
     c1 = @http.connection_for uri1
 
